@@ -47,6 +47,8 @@ from docutils.frontend import Values
 from docutils.nodes import block_quote
 from docutils.nodes import citation
 from docutils.nodes import field_body
+from docutils.nodes import footnote
+from docutils.nodes import footnote_reference
 from docutils.nodes import list_item
 
 
@@ -107,7 +109,7 @@ class Model(docutils.nodes.GenericNodeVisitor):
         )
         self.log.frame = [
             "{now}", "{level.name:>8}", "{logger.name}",
-            "{1[path]}", "{1[line_nr]:>5}", " {0}", " {token}"
+            "{1[path]}", "{1[line_nr]:>5}", " {0:<64}", " {token}"
         ]
 
         self.section_level = 0
@@ -165,7 +167,7 @@ class Model(docutils.nodes.GenericNodeVisitor):
                 {"path": self.fP, "line_nr": line},
                 token=matchObj.group(1)
             )
-            rv = ""
+            rv = None
         return rv
 
     def default_visit(self, node):
@@ -274,9 +276,38 @@ class Model(docutils.nodes.GenericNodeVisitor):
 
     def depart_field_body(self, node):
         name, _ = self.metadata.pop(-1)
-        self.metadata.append((name, " ".join(self.text)))
+        if self.text:
+            self.metadata.append((name, " ".join(self.text)))
         self.text.clear()
 
+    def depart_footnote_reference(self, node):
+        try:
+            span = self.html.pop(-1)
+            self.html.append(span.replace('class="text"','class="call"'))
+        except InderError:
+            self.log.warning(
+                "Unable to process footnote callout",
+                {"path": self.fP, "line_nr": node.line},
+            )
+
+    def visit_footnote(self, node):
+        self.text = []
+        self.html.append('<span class="footnote" role="note">')
+
+    def depart_footnote(self, node):
+        try:
+            for n, span in enumerate(self.html.copy()):
+                if '<span class="text">' in span:
+                    self.html[n] = span.replace('<span class="text">', "").replace("</span>", "")
+        except InderError:
+            self.log.warning(
+                "Unable to process footnote",
+                {"path": self.fP, "line_nr": node.line},
+            )
+        self.html.append("</span>")
+        self.html.append("</p>\n")
+        self.close_shot(node.line)
+
     def visit_list_item(self, node):
         self.html.append("<li>")
 
@@ -292,12 +323,19 @@ class Model(docutils.nodes.GenericNodeVisitor):
         ))
 
     def visit_paragraph(self, node):
-        if self.shots and not isinstance(node.parent, (citation, field_body, list_item)):
+        if self.shots and not isinstance(node.parent, (citation, field_body, footnote, list_item)):
             self.text = []
             self.html = ["<p>"]
 
     def depart_paragraph(self, node):
-        if self.shots and not isinstance(node.parent, (citation, field_body, list_item)):
+        if any(isinstance(i, footnote_reference) for i in node.children):
+            self.log.debug(
+                "Not closing after footnote reference",
+                {"path": self.fP, "line_nr": node.line},
+            )
+            return
+
+        if self.shots and not isinstance(node.parent, (citation, field_body, footnote, list_item)):
             self.html.append("</p>\n")
             self.close_shot(node.line)
 
@@ -384,12 +422,14 @@ class Model(docutils.nodes.GenericNodeVisitor):
                 entity = self.get_entity(ref)
                 if entity is None:
                     obj = Pathfinder.string_import(
-                        tgt["arguments"][0], relative=False, sep="."
-                    )
-                    self.text.append(str(obj).strip())
-                    self.html.append(
-                        str(obj).strip().translate(self.escape_table)
+                        tgt["arguments"][0], relative=False, sep=".",
+                        path=self.fP, line_nr=defn.line
                     )
+                    if obj is not None:
+                        self.text.append(str(obj).strip())
+                        self.html.append(
+                            str(obj).strip().translate(self.escape_table)
+                        )
                 elif getattr(entity, "persona", None) is not None:
                     fmt = "".join(("{0.", attr, "}"))
                     val = fmt.format(entity.persona)
